@@ -1,4 +1,8 @@
 import asyncio
+import threading
+
+_cached_loop = threading.local()
+"""Loop that will be used when the SM is running in a synchronous context. One loop per thread."""
 
 
 def qualname(cls):
@@ -23,11 +27,13 @@ def ensure_iterable(obj):
 
 def run_async_from_sync(coroutine):
     """
-    Run an async coroutine from a synchronous context.
+    Compatibility layer to run an async coroutine from a synchronous context.
     """
+    global _cached_loop
     try:
-        loop = asyncio.get_running_loop()
-        return asyncio.ensure_future(coroutine)
+        asyncio.get_running_loop()
+        return coroutine
     except RuntimeError:
-        loop = asyncio.get_event_loop()
-        return loop.run_until_complete(coroutine)
+        if not hasattr(_cached_loop, "loop"):
+            _cached_loop.loop = asyncio.new_event_loop()
+        return _cached_loop.loop.run_until_complete(coroutine)
